@@ -6,6 +6,7 @@ housekeeping <- function(e, ...)UseMethod("housekeeping")
 inProgressMenu <- function(e, choices, ...)UseMethod("inProgressMenu")
 courseMenu <- function(e, courses)UseMethod("courseMenu")
 courseDir <- function(e)UseMethod("courseDir")
+progressDir <- function(e)UseMethod("progressDir")
 lessonMenu <- function(e, choices)UseMethod("lessonMenu")
 restoreUserProgress <- function(e, selection)UseMethod("restoreUserProgress")
 loadLesson <- function(e, ...)UseMethod("loadLesson")
@@ -26,7 +27,7 @@ mainMenu.default <- function(e){
   # Welcome the user if necessary and set up progress tracking
   if(!exists("usr",e,inherits = FALSE)){
     e$usr <- welcome(e)
-    udat <- file.path(find.package("swirl"), "user_data", e$usr)
+    udat <- file.path(progressDir(e), e$usr)
     if(!file.exists(udat)){
       housekeeping(e)
       dir.create(udat, recursive=TRUE)
@@ -58,15 +59,10 @@ mainMenu.default <- function(e){
       
       # If no courses are available, offer to install one
       if(length(coursesU)==0){
-        suggestions <- yaml.load_file(file.path(courseDir(e), "suggested_courses.yaml"))
+        suggestions <- yaml.load_file(file.path(find.package("swirl"), "Courses", "suggested_courses.yaml"))
         choices <- sapply(suggestions, function(x)paste0(x$Course, ": ", x$Description))
-        swirl_out("To begin, you must install a course. I can install a",
-                  "course for you from the internet, or I can send you to a web page",
-                  "(https://github.com/swirldev/swirl_courses)",
-                  "which will provide course options and directions for", 
-                  "installing courses yourself.",
-                  "(If you are not connected to the internet, type 0 to exit.)")
-        choices <- c(choices, "Don't install anything for me. I'll do it myself.")
+        swirl_out(s()%N%"To begin, you must install a course. I can install a course for you from the internet, or I can send you to a web page (https://github.com/swirldev/swirl_courses) which will provide course options and directions for installing courses yourself. (If you are not connected to the internet, type 0 to exit.)")
+        choices <- c(choices, s()%N%"Don't install anything for me. I'll do it myself.")
         choice <- select.list(choices, graphics=FALSE)
         n <- which(choice == choices)
         if(length(n) == 0)return(FALSE)
@@ -74,17 +70,17 @@ mainMenu.default <- function(e){
           repeat {
             temp <- try(eval(parse(text=suggestions[[n]]$Install)), silent=TRUE)
             if(is(temp, "try-error")){
-              swirl_out("Sorry, but I'm unable to fetch ", sQuote(choice),
-                        "right now. Are you sure you have an internet connection?",
-                        "If so, would you like to try again or visit",
-                        "the course repository for instructions on how to",
-                        "install a course manually? Type 0 to exit.")
-              ch <- c("Try again!", 
-                      "Send me to the course repository for manual installation.")
+              swirl_out(s()%N%"Sorry, but I'm unable to fetch ", sQuote(choice),
+                        s()%N%"right now. Are you sure you have an internet connection?",
+                        s()%N%"If so, would you like to try again or visit",
+                        s()%N%"the course repository for instructions on how to",
+                        s()%N%"install a course manually? Type 0 to exit.")
+              ch <- c(s()%N%"Try again!", 
+                      s()%N%"Send me to the course repository for manual installation.")
               resp <- select.list(ch, graphics=FALSE)
               if(resp == "") return(FALSE)
               if(resp == ch[2]) {
-                swirl_out("OK. I'm opening the swirl course respository in your browser.")
+                swirl_out(s()%N%"OK. I'm opening the swirl course respository in your browser.")
                 browseURL("https://github.com/swirldev/swirl_courses")
                 return(FALSE)
               }
@@ -93,12 +89,20 @@ mainMenu.default <- function(e){
             }
           }
           coursesU <- dir(courseDir(e))
+          if(length(coursesU) > 0){
+            for(i in 1:length(coursesU)){
+              coursesU[i] <- enc2utf8(coursesU[i])
+            }
+          }
+          if(any(is.na(coursesU))){
+            coursesU <- dir(courseDir(e))
+          }
           # Eliminate empty directories
           idx <- unlist(sapply(coursesU, 
                                function(x)length(dir(file.path(courseDir(e),x)))>0))
           coursesU <- coursesU[idx]
         } else {
-          swirl_out("OK. I'm opening the swirl course respository in your browser.")
+          swirl_out(s()%N%"OK. I'm opening the swirl course respository in your browser.")
           browseURL("https://github.com/swirldev/swirl_courses")
           return(FALSE)
         }
@@ -109,7 +113,7 @@ mainMenu.default <- function(e){
       while(lesson == ""){
         course <- courseMenu(e, coursesR)
         if(!is.null(names(course)) && names(course)=="repo") {
-          swirl_out("OK. I'm opening the swirl courses web page in your browser.")
+          swirl_out(s()%N%"OK. I'm opening the swirl courses web page in your browser.")
           browseURL("https://github.com/swirldev/swirl_courses")
           return(FALSE)
         }
@@ -119,6 +123,7 @@ mainMenu.default <- function(e){
         # reverse path cosmetics
         courseU <- coursesU[course == coursesR]
         course_dir <- file.path(courseDir(e), courseU)
+        
         # Get all files/folders from course dir, excluding MANIFEST
         lessons <- dir(course_dir)
         lessons <- lessons[lessons != "MANIFEST"]
@@ -129,6 +134,21 @@ mainMenu.default <- function(e){
           lessons <- order_lessons(current_order=lessons, 
                                    manifest_order=manifest)
         }
+        # If the manifest introduced NAs, try reading without UTF-8
+        if(any(is.na(lessons))){
+          manifest <- get_manifest(course_dir, utf8 = FALSE)
+          lessons <- order_lessons(current_order=lessons,
+                                   manifest_order=manifest)
+        }
+        # If there are still NAs, throw the manifest out
+        if(any(is.na(lessons))){
+          lessons <- list.dirs(course_dir, full.names = FALSE, recursive = FALSE)
+          # Get rid of hidden folders if they exist
+          if(length(grep("^\\.", lessons)) > 0){
+            lessons <- lessons[-grep("^\\.", lessons)]
+          }
+        }
+        
         # Clean up lesson names
         lessons_clean <- gsub("_", " ", lessons)
         # Let user choose the lesson.
@@ -154,7 +174,7 @@ mainMenu.default <- function(e){
       e$path <- file.path(courseDir(e), courseU, lesson)
       # If running in 'test' mode and starting partway through 
       # lesson, then complete first part
-      if(is(e, "test") && e$test_from > 1) {
+      if((is(e, "test") || is(e, "datacamp")) && e$test_from > 1) {
         complete_part(e)
       }
       
@@ -163,12 +183,14 @@ mainMenu.default <- function(e){
       rm("temp_lesson_name", "temp_course_name", envir=e, inherits=FALSE)
       
       # Initialize the progress bar
-      e$pbar <- txtProgressBar(style=3)
+      if(!is(e,"datacamp")) {
+        e$pbar <- txtProgressBar(style=3)
+      }
       e$pbar_seq <- seq(0, 1, length=nrow(e$les))
       
       # expr, val, ok, and vis should have been set by the callback.
       # The lesson's current row - could start after 1 if in 'test' mode
-      if(is(e, 'test')) {
+      if(is(e, 'test') || is(e, 'datacamp')) {
         e$row <- e$test_from
       } else {
         e$row <- 1
@@ -189,8 +211,23 @@ mainMenu.default <- function(e){
       e$progress <- file.path(e$udat, fname)
       # indicator that swirl is not reacting to console input
       e$playing <- FALSE
+      
+      # Create log
+      if(isTRUE(getOption("swirl_logging"))){
+        e$log <- list(user = e$usr, 
+                      course_name = attr(e$les,"course_name"),
+                      lesson_name = attr(e$les,"lesson_name"),
+                      question_number = NULL,
+                      correct = NULL,
+                      attempt = NULL,
+                      skipped = NULL,
+                      datetime = NULL)
+      }
+
       # create the file
       suppressMessages(suppressWarnings(saveRDS(e, e$progress)))
+      # post initialization message
+      post_init(e)
     }
   }
   return(TRUE)
@@ -201,14 +238,14 @@ welcome.test <- function(e, ...){
 }
 
 # Default version.
+#' @importFrom stringr str_detect str_trim
 welcome.default <- function(e, ...){
-  swirl_out("Welcome to swirl!")
-  swirl_out("Please sign in. If you've been here before, use the same name as you did then. If you are new, call yourself something unique.", skip_after=TRUE)
-  resp <- readline("What shall I call you? ")
-  while(str_detect(resp, '[[:punct:]]')) {
-    swirl_out("Please don't use any quotes or other punctuation in your name.",
+  swirl_out(s()%N%"Welcome to swirl! Please sign in. If you've been here before, use the same name as you did then. If you are new, call yourself something unique.", skip_after=TRUE)
+  resp <- readline(s()%N%"What shall I call you? ")
+  while(str_detect(resp, '[[:punct:]]') || nchar(str_trim(resp)) < 1) {
+    swirl_out(s()%N%"Please don't use any quotes or other punctuation in your name.",
               skip_after = TRUE)
-    resp <- readline("What shall I call you? ")
+    resp <- readline(s()%N%"What shall I call you? ")
   }
   return(resp)
 }
@@ -218,14 +255,14 @@ welcome.default <- function(e, ...){
 # @param e persistent environment used here only for its class attribute
 # 
 housekeeping.default <- function(e){
-  swirl_out(paste0("Thanks, ", e$usr,". Let's cover a few quick housekeeping items before we begin our first lesson. First of all, you should know that when you see '...', that means you should press Enter when you are done reading and ready to continue."))
-  readline("\n...  <-- That's your cue to press Enter to continue")
-  swirl_out("Also, when you see 'ANSWER:', the R prompt (>), or when you are asked to select from a list, that means it's your turn to enter a response, then press Enter to continue.")
-  select.list(c("Continue.", "Proceed.", "Let's get going!"),
-              title="\nSelect 1, 2, or 3 and press Enter", graphics=FALSE)
-  swirl_out("You can exit swirl and return to the R prompt (>) at any time by pressing the Esc key. If you are already at the prompt, type bye() to exit and save your progress. When you exit properly, you'll see a short message letting you know you've done so.")
+  swirl_out(paste0(s()%N%"Thanks, ", e$usr, s()%N%". Let's cover a couple of quick housekeeping items before we begin our first lesson. First of all, you should know that when you see '...', that means you should press Enter when you are done reading and ready to continue."))
+  readline(s()%N%"\n...  <-- That's your cue to press Enter to continue")
+  swirl_out(s()%N%"Also, when you see 'ANSWER:', the R prompt (>), or when you are asked to select from a list, that means it's your turn to enter a response, then press Enter to continue.")
+  select.list(c(s()%N%"Continue.", s()%N%"Proceed.", s()%N%"Let's get going!"),
+              title=s()%N%"\nSelect 1, 2, or 3 and press Enter", graphics=FALSE)
+  swirl_out(s()%N%"You can exit swirl and return to the R prompt (>) at any time by pressing the Esc key. If you are already at the prompt, type bye() to exit and save your progress. When you exit properly, you'll see a short message letting you know you've done so.")
   info()
-  swirl_out("Let's get started!", skip_before=FALSE)
+  swirl_out(s()%N%"Let's get started!", skip_before=FALSE)
   readline("\n...")
 }
 
@@ -233,8 +270,8 @@ housekeeping.test <- function(e){}
 
 # A stub. Eventually this should be a full menu
 inProgressMenu.default <- function(e, choices){
-  nada <- "No. Let me start something new."
-  swirl_out("Would you like to continue with one of these lessons?")
+  nada <- s()%N%"No. Let me start something new."
+  swirl_out(s()%N%"Would you like to continue with one of these lessons?")
   selection <- select.list(c(choices, nada), graphics=FALSE)
   # return a blank if the user rejects all choices
   if(identical(selection, nada))selection <- ""
@@ -247,9 +284,9 @@ inProgressMenu.test <- function(e, choices) {
 
 # A stub. Eventually this should be a full menu
 courseMenu.default <- function(e, choices){
-  repo_option <- "Take me to the swirl course repository!"
+  repo_option <- s()%N%"Take me to the swirl course repository!"
   choices <- c(choices, repo = repo_option)
-  swirl_out("Please choose a course, or type 0 to exit swirl.")
+  swirl_out(s()%N%"Please choose a course, or type 0 to exit swirl.")
   return(select.list(choices, graphics=FALSE))
 }
 
@@ -259,7 +296,7 @@ courseMenu.test <- function(e, choices) {
 
 # A stub. Eventually this should be a full menu
 lessonMenu.default <- function(e, choices){
-  swirl_out("Please choose a lesson, or type 0 to return to course menu.")
+  swirl_out(s()%N%"Please choose a lesson, or type 0 to return to course menu.")
   return(select.list(choices, graphics=FALSE))
 }
 
@@ -359,8 +396,12 @@ completed <- function(e){
   return(pfiles)
 }
 
-get_manifest <- function(course_dir) {
-  man <- readLines(file.path(course_dir, "MANIFEST"), warn=FALSE)
+get_manifest <- function(course_dir, utf8 = TRUE) {
+  if(utf8){
+    man <- readLines(file.path(course_dir, "MANIFEST"), warn=FALSE, encoding = "UTF-8")
+  } else {
+    man <- readLines(file.path(course_dir, "MANIFEST"), warn=FALSE)
+  }
   # Remove leading and trailing whitespace
   man <- str_trim(man)
   # Remove empty lines
@@ -375,7 +416,11 @@ order_lessons <- function(current_order, manifest_order) {
 
 courseDir.default <- function(e){
   # e's only role is to determine the method used
-  file.path(find.package("swirl"), "Courses")
+  swirl_courses_dir()
+}
+
+progressDir.default <- function(e) {
+  swirl_data_dir()
 }
 
 # Default for determining the user
